{% extends "svnlit/node_base.html" %}

{% block node_content %}

<ul class="node-list">
  {% if not node.is_root %}
  <li class="parent-directory"><a href="{{ node.parent.get_absolute_url }}">../</a></li>
  {% else %}
  <li class="parent-directory"><a href="{% url svnlit_repository_list %}">../</a></li>
  {% endif %}

  {% for child in node.children.all %}
  <li class="{% if child.is_directory %}directory{% else %}file{% endif %}">
    <a href="{{ child.get_absolute_url }}">
      {{ child.get_basename }}{% if child.is_directory %}/{% endif %}</a>
  </li>
  {% endfor %}
</ul>

{% endblock %}
