<span class="node-path">
  {% for path, basename in node.iter_path_basename reversed %}
  <span class="node">
    {% if forloop.last %}
    {{ basename }}
    {% else %}
    {% ifnotequal repository.get_latest_revision revision %}
    <a href="{% url svnlit_node_revision repository.label,revision,path %}">
      {{ basename }}</a>
    {% else %}
    <a href="{% url svnlit_node repository.label,path %}">{{ basename }}</a>
    {% endifnotequal %}
    {% endif %}
  </span>
  {% if not forloop.last %}<span class="sep">/</span>{% endif %}
  {% endfor %}
</span>
